/*
 *    snake - a videogame derived from Q-Basic nibbles
 *    Copyright (C) 1999-2000 Cyberandro J.K. Starrik
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include "snake.h"
#include <string.h>		/* strcat, ...   */
#include <errno.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>		/* getpwuid, ...   */
#include <sys/stat.h>		/* stat, ...   */


/* public functions */
int register_command (char *, int (*)(char *));
int parse_command (char *);
#ifdef ENABLE_CMDFIFO
void init_cmdfifo (void);
void read_cmdfifo ();
#endif

/* public data */
#ifdef ENABLE_CMDFIFO
FILE *cmdfifo;
char *cmdfifofile;
#endif

/* private functions */
static int eval_command (char *, char *);

/*
 * local data 
 */
struct t_command_table
{
  char label[MAXCMDLBLLEN];
  int (*handler) (char *);
  struct t_command_table *next;
}
 *command_table;


/*
 * register a command handler in the command table 
 */
int
register_command (char *label, int (*handler) (char *))
{
  struct t_command_table *ptr, *lptr;

  /*
   * scan the command table 
   */
  for (lptr = ptr = command_table; ptr; lptr = ptr, ptr = ptr->next)
    {

      /*
       * command already registered 
       */
      if (!strcmp (ptr->label, label))
	return 0;

    }

  /*
   * append the new command if not already registered 
   */
  ptr =
    safe_malloc (sizeof (struct t_command_table), "new command table item");
  strncpy (ptr->label, label, MAXCMDLBLLEN - 1);
  ptr->handler = handler;
  ptr->next = NULL;

  if (lptr)
    lptr->next = ptr;
  else
    command_table = ptr;

#ifdef DEBUG_CONFIG
  message ("command '%s' registered", label);
#endif

  return 1;
}

/*
 * evaluate a command with the given argument 
 */
static int
eval_command (char *label, char *arg)
{
  struct t_command_table *ptr;

  /*
   * scan the command table 
   */
  for (ptr = command_table; ptr; ptr = ptr->next)
    if (!strcmp (ptr->label, label))
      return ptr->handler (arg);

  /*
   * unknown command 
   */
  message ("Unknown command: %s", label);
  return 0;
}

/* parse a single command line */
int
parse_command (char *s)
{
  char *command;

  /*
   * this is a comment 
   */
  if (s[0] == '#')
    return 0;

  command = strtok (s, " \t\n");
  if (command)
    return eval_command (command, strtok (NULL, "\n"));

  return 1;
}


#ifdef ENABLE_CMDFIFO
void
init_cmdfifo ()
{

  cmdfifo = fopen (cmdfifofile, "r+");
  if (!cmdfifo)
    {
      message ("Error opening command fifo %s: %s",
	       cmdfifofile, strerror (errno));
      return;
    }

  /* set nonblocking read mode */
  if (fcntl (fileno (cmdfifo), F_SETFL, O_NONBLOCK) == -1)
    perror ("fcntl");

}

void
read_cmdfifo ()
{
  char buf[256];

  if (fgets (buf, 256, cmdfifo))
    parse_command (buf);
}

int
CMD_cmdfifo (char *arg)
{
  FILE *newcmdfifo;

  if (!arg)
    {
      message ("CMD_cmdfifo: syntax error");
      return 0;
    }

  newcmdfifo = fopen (arg, "r+");
  if (!newcmdfifo)
    {
      message ("Error opening command fifo %s: %s", arg, strerror (errno));
      return 0;
    }

  /* set nonblocking read mode */
  if (fcntl (fileno (newcmdfifo), F_SETFL, O_NONBLOCK) == -1)
    perror ("fcntl");

  fclose (cmdfifo);
  cmdfifo = newcmdfifo;
  cmdfifofile = arg;

  message ("Command fifo: %s", cmdfifofile);

  return 1;
}
#endif /* ENABLE_CMDFIFO */
