/*
 *    snake - a videogame derived from Q-Basic nibbles
 *    Copyright (C) 1999-2000 Cyberandro J.K. Starrik
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

/*
 * file: snake_curses.c *  purpose: handle curses graphics 
 */

#include "snake.h"
#include <curses.h>
#include <stdlib.h>

#ifdef HAVE_READLINE_H
#include <readline.h>
#elifdef HAVE_READLINE_READLINE_H
#include <readline/readline.h>
#endif

#define PROMPT_STRING "*--(eval): "

/*
 * public functions 
 */
int init_graphics (void);
void draw_arena (void);
void draw_message (char *msg);
void draw_motion (void);
void draw_delay (int delay);
void draw_fruit_position (int y, int x);
void updatescreen (void);

/*
 * private functions 
 */
static int init_screen (void);
static void make_arena (void);
static void make_prompt (void);

/*
 * Public data
 */
int screen_initialized = 0;
WINDOW *arenawin;
WINDOW *promptwin;

/*
 * initialize the graphics 
 */
int
init_graphics ()
{
  extern int delay;

  /*
   * init screen 
   */
  if (!init_screen ())
    {
      fputs ("can't initialize graphic\n", stderr);
      exit (1);
    }

  /*
   * make arena window 
   */
  make_arena ();

  /*
   * make prompt window
   */
  make_prompt ();

  /*
   * print delay string 
   */
  draw_delay (delay);

  /* Set screen initialized flag */
  screen_initialized = 1;

  return 1;

}

/*
 * curses version of drawing function 
 */
void
draw_message (char *str)
{
  int y, x;

  mvwaddstr (arenawin, LINES - 1, 5, "* ");
  waddstr (arenawin, str);
  waddstr (arenawin, " *");
  do
    {
      getyx (arenawin, y, x);
      waddch (arenawin, '-');
    }
  while (x < ARENA_WIDTH);

}

/*
 * curses version of arena drawing 
 */
void
draw_arena ()
{
  int y, x;
  sprite *sp;
  chtype look;
  int lines, cols;

  lines = arenalines ();
  cols = arenacols ();

  for (y = 0; y < lines; y++)
    {
      for (x = 0; x < cols; x++)
	{
	  sp = arenaget (y, x);
	  if (sp)
	    switch (sp->type)
	      {

	      case SNAKE:
		look =
		  (((snakebody *) (sp))->self->playernum + 48) | A_REVERSE;
		break;

	      case EXSNAKE:
		look = '.';
		break;

	      case FRUIT:
		look = ((fruit *) (sp))->value;
		break;

	      case WALL:
		look = '#';
		break;

	      default:
		look = '?';
		break;

	      }

	  else
	    look = ' ';

	  mvwaddch (arenawin, y + ARENA_Y_OFFSET, x + ARENA_X_OFFSET, look);
	}
    }

}

/*
 * activity widget 
 */
void
draw_motion (void)
{
  static char sprite[] = { '-', '/', '|', '\\' };
  static int spriteidx = 0;

  if (++spriteidx == 4)
    spriteidx = 0;
  mvwaddch (arenawin, 0, 0, sprite[spriteidx]);
}

/*
 * draw delay string 
 */
void
draw_delay (int delay)
{
  char str[20];

  /*
   * write motion string 
   */
  snprintf (str, 19, " delay: %3.3i ", delay);
  mvwaddstr (arenawin, 0, 5, str);
}

/*
 * draw fruit position 
 */
void
draw_fruit_position (int y, int x)
{
  char str[18];

  /*
   * write fruit position 
   */
  sprintf (str, " fruit: %2.2i, %2.2i ", y, x);
  mvwaddstr (arenawin, 0, ARENA_WIDTH - 20, str);
}

/*
 * update the screen 
 */
void
updatescreen ()
{
  touchwin (arenawin);
  wrefresh (arenawin);
}


/************ PRIVATE FUNCTIONS **********************/

/*
 * initialize the screen  
 */
static int
init_screen ()
{
  /*
   * initialize the COLS and LINES vars 
   */
  initscr ();

  /*
   * make cursor invisible 
   */
  if (curs_set (0) == ERR)
    return 0;

  /*
   * don't echo input characthers 
   */
  if (noecho () == ERR)
    return 0;

  /*
   * make each carachter break input reads 
   */
  if (cbreak () == ERR)
    return 0;

  /*
   * make getch() be a non-blocking call  
   */
  if (nodelay (stdscr, TRUE) == ERR)
    return 0;

  /*
   * redraw completely on refresh 
   */
  clearok (stdscr, TRUE);

  /*
   * success 
   */
  return 1;
}

/*
 * make prompt window
 */
static void
make_prompt ()
{
  promptwin = newwin (1, COLS, LINES - 1, 0);
}

/*
 * make arena window 
 */
static void
make_arena ()
{
  /*
   * make arena window 
   */
  arenawin = newwin (0, 0, 0, 0);
  wborder (arenawin, '|', '|', '-', '-', '*', '*', '*', '*');
}

/*
 * prompt the user for a string
 * the line returnered is allocated with malloc(3)
 * so the caller must free it when finished with it. 
 */
char *
prompt ()
{
  static char *cmd;

  curs_set (1);			/* show cursor */

#ifdef WITH_READLINE

  /* show prompt window */
  touchwin (promptwin);
  wrefresh (promptwin);

  reset_shell_mode ();
  cmd = readline (PROMPT_STRING);
  reset_prog_mode ();

#else
  cmd = safe_malloc (MAXCMDLINELEN, "prompt input buffer");

  echo ();
  werase (promptwin);
  mvwaddstr (promptwin, 0, COLS - 5, "*---*");
  mvwaddstr (promptwin, 0, 0, PROMPT_STRING);
  touchwin (promptwin);
  wrefresh (promptwin);
  wgetnstr (promptwin, cmd, COLS - strlen (PROMPT_STRING) - 6);
  noecho ();
#endif

  curs_set (0);			/* hide cursor */

  /* standard screen is probabily scrolled */
  touchwin (stdscr);

  return cmd;
}
