/*
 *    snake - a videogame derived from Q-Basic nibbles
 *    Copyright (C) 1999-2000 Cyberandro J.K. Starrik
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include "snake.h"
#include <stdlib.h>		/* getenv, ...   */
#include <time.h>		/* time (for srand) */
#include <sys/time.h>		/* itimer ...   */
#include <glob.h>		/* glob_t etc.   */
#include <string.h>		/* strcat, strcmp, ...   */
#include <dlfcn.h>		/* dlopen, dlclose, ...   */
#include <setjmp.h>		/* setjmp, longjmp */
#include <signal.h>		/* signal */

/*
 * extern functions 
 */
extern int init_graphics (void);
extern void move_snakes (void);
extern void init_fruit (void);
extern int parse_input (void);
extern void draw_motion (void);
extern void print_delay (void);
extern void open_config (void);
extern void close_config (void);
extern void init_modules (void);
extern void draw_arena (void);
extern int parse_cmdline_opts (int, char **);
extern void parse_cmdline_args (int, char **);
extern int parse_config ();
extern int CMD_delay (char *);
extern int CMD_kill (char *);
extern int CMD_reset (char *);
extern int CMD_trim (char *);
extern void expire_messages (void);

#ifdef ENABLE_CMDFIFO
extern int CMD_cmdfifo (char *);
extern void init_cmdfifo (void);
extern void read_cmdfifo (void);
extern FILE *cmdfifo;
#endif

/* extern data */
extern WINDOW *arenawin;

/*
 * private functions 
 */
static void snake_play (void);
static void siginthandler (int sig);
#ifdef PLAYERTIMEOUT
static void player_timeout (int sig);
#endif
static int CMD_quit (char *);

/*
 * public function 
 */
int register_player (snake * s);
void finish (void);

/*
 * public data 
 */
snake *zoo[MAXPLAYERS];		/* snakes pointer */
int numplayers = 0;		/* number of players */
int delay = 40;			/* game delay in milliseconds */
unsigned long int turns = 0;	/* game turns */


int
main (int argc, char **argv)
{
  int optind;

  /* register core commands */
  register_command ("delay", CMD_delay);
#ifdef ENABLE_CMDFIFO
  register_command ("cmdfifo", CMD_cmdfifo);
#endif
  register_command ("kill", CMD_kill);
  register_command ("reset", CMD_reset);
  register_command ("trim", CMD_trim);
  register_command ("quit", CMD_quit);

  /* parse command line options */
  optind = parse_cmdline_opts (argc, argv);

  /* initialize the graphics */
  init_graphics ();

  /* set signal handlers */
  signal (SIGINT, siginthandler);

#ifdef PLAYERTIMEOUT
  signal (SIGVTALRM, player_timeout);	/* players timeout */
  signal (SIGALRM, player_timeout);	/* players timeout */
#endif

  /*
   * init the arena 
   */
  arenainit (ARENA_HEIGHT, ARENA_WIDTH);

  /*
   * initialize the random seed 
   */
  srand (time (NULL));

  /*
   * initialize the snakes 
   */
  init_modules ();

  /*
   * parse configuration file 
   */
  parse_config ();

  /*
   * parse command line arguments
   * parse it after configuration file
   * to override values specified there
   */
  parse_cmdline_args (argc - optind, argv + optind);

  /* initialize the fruit */
  init_fruit ();

  /* update the screen */
  updatescreen ();

  /* MAIN LOOP  */
  for (;;)
    {
      /* parse keyboard input */
      parse_input ();

#ifdef ENABLE_CMDFIFO
      /* Read from command fifo */
      if (cmdfifo)
	read_cmdfifo ();
#endif

      /* give players a turn to play */
      snake_play ();

      /* move the snakes */
      move_snakes ();

      /* draw the arena */
      draw_arena ();

      /* show activity */
      draw_motion ();

      /* update the screen with the new data */
      updatescreen ();

      /* sleep for delay milliseconds (could use setitimer) */
      napms (delay);

      /* increment bogus time counter */
      turns++;

      expire_messages ();

    }

  finish ();
  exit (0);			/* finish will exit itself */
}

void
finish ()
{
  int i;

  /*
   * reset the screen 
   */
  endwin ();

  printf ("\n");

  /*
   * print scores 
   */
  for (i = 0; zoo[i]; i++)
    {
      printf ("snake %i: score(%li), length(%i)\n",
	      i, zoo[i]->score, zoo[i]->length);
    }

  exit (0);
}


#ifdef PLAYERTIMEOUT
jmp_buf playloop;
#endif

static void
snake_play (void)
{
  int i;
#ifdef PLAYERTIMEOUT
  static struct itimerval playtimer = {
    {0, PLAYERTIMEOUT},		/*
				 * it_interval.{tv_sec,tv_usec} 
				 */
    {0, PLAYERTIMEOUT},		/*
				 * it_value.{tv_sec,tv_usec} 
				 */
  };
#endif


  for (i = 0; zoo[i]; i++)
    {

#ifdef PLAYERTIMEOUT
      if (setjmp (playloop))
	{

	  /*
	   * reset timer 
	   */
	  setitimer (TIMERMODE, NULL, NULL);

	  message ("!! snake %i thinks too much !!", i);

	  continue;
	}
#endif

      if (zoo[i]->died)
	continue;

      /*
       * rewind typein queue 
       */
      rewindtypein ();

#ifdef PLAYERTIMEOUT
      /*
       * start timer 
       */
      setitimer (TIMERMODE, &playtimer, NULL);
#endif

      /*
       * call player function 
       */
      (*zoo[i]->play) (zoo[i]);

#ifdef PLAYERTIMEOUT
      /*
       * remove timer 
       */
      setitimer (TIMERMODE, NULL, NULL);
#endif

    }
}

#ifdef PLAYERTIMEOUT
static void
player_timeout (int sig)
{
  /*
   * reset signal handler (linux needs this, bsd doesn't) 
   */
  signal (sig, player_timeout);
  longjmp (playloop, 1);
}
#endif


/*
 * player modules must call this function at startup 
 */
int
register_player (snake * s)
{
  /*
   * don't register more then MAXPLAYERS snakes ! 
   * (gotta remove this limitation)
   */
  if (numplayers >= MAXPLAYERS - 1)
    {
      message ("maximum count of players reached");
      return 0;
    }

  /*
   * register new player 
   */
  zoo[numplayers] = s;

  /*
   * set numplayers flag 
   */
  zoo[numplayers]->playernum = numplayers;

  message ("player %d registered", numplayers);

  return numplayers++;
}

static void
siginthandler (int sig)
{
  finish ();
}

static int
CMD_quit (char *dummy)
{
  finish ();
  return 1;
}
